# -*- coding: utf8 -*-

from qt import *
from Queue import Queue

from kittywidget import *
from utils import *
from ui import ICONPATH
from bbs import Board


class TopicListWidget(KittyTabWidget):
    """Widget to display topic list"""

    _tab_menu_spec = ((u("更新"),   "requestReload"),
                      (u("閉じる"), "requestClose"))

    def __init__(self, parent):
        KittyTabWidget.__init__(self, parent)
        self._context_menu = self._ContextMenu(self)
        self._newtab = True
        self.connect(self._tab_menu, PYSIGNAL("requestClose"), self._close)
        self.connect(self._tab_menu, PYSIGNAL("requestReload"),
                     self._reload_pointed_page)
        self.connect(self,
                     PYSIGNAL("tabMidPressed"),
                     self._close_pointed_page)
        self.connect(self,
                     PYSIGNAL("tabRightPressed"),
                     self._tab_menu.popup)
        self.connect(self,
                     PYSIGNAL("tabLeftDoubleClicked"),
                     self._reload_current_page)
        self.connect(self._context_menu, PYSIGNAL("requestOpenInNewTab"),
                     self._request_open_in_new_tab)
        self.connect(self._context_menu, PYSIGNAL("requestOpenInCurrentTab"),
                     self._request_open_in_current_tab)
        self.connect(self._context_menu, PYSIGNAL("requestBookmark"),
                     self._request_bookmark)

    def display(self, board, topics, newtab=None):
        """Interface to display the topic list of board
        
        This method actually submit board into the local queue.  If 
        newtab is True, the new tab will be created for this board.
        """
        assert isinstance(board, Board)
        if newtab==None:
            newtab = self._newtab
        self._queue.put((board, topics, newtab))

    def close_current_page(self):
        """Close currently displayed page.

        If no page is open, thid method does nothing.
        """
        w = self.currentPage()
        if w!=None:
            self._close(w)

    def current_board(self):
        widget = self.currentPage()
        return self._model_view_registry.model(widget)

    def _request_open_in_new_tab(self, item):
        self._request_open(item, True)

    def _request_open_in_current_tab(self, item):
        self._request_open(item, False)

    def _request_open(self, item, newtab=None):
        """Request to open given item

        This method emits requestOpen(topic) pysignal.
        """
        if newtab is None:
            newtab = self._newtab

        self.emit(PYSIGNAL("requestOpenMessages"), (item.topic(), newtab))

    def _request_bookmark(self, item):
        self.emit(PYSIGNAL("requestAppendBookmark"), (item.topic(),))

    def _display(self, board, topics, newtab):
        """Display the topic list of board, used from sub thread.
        
        If this window is already displaying the board or newtab is 
        False, the tab is  shown.  Otherwise, create a new tab and 
        display the list in the tab.
        """
        if board in self._model_view_registry.models():
            listview = self._model_view_registry.view(board)
        elif newtab==False and self._model_view_registry.size()>0:
            listview = self.currentPage()
            self.changeTab(listview, board.title())
            self._model_view_registry.remove(self.currentPage())
        else:
            listview = self._create_list_view()
            self.addTab(listview, board.title())

        listview.clear()
        i = 1
        for topic in topics:
            self._ListViewItem(topic,
                               listview,
                               "%03d"%i,
                               topic.title(),
                               "%d" % topic.remote_size(),
                               "%d" % (topic.local_size() or 0))
            i += 1
        self._model_view_registry.set(board, listview)
        self.showPage(self._model_view_registry.view(board))

    def _popup_context_menu(self, item, position, column):
        self._context_menu.popup(item, position)

    def _close_pointed_page(self, page, event):
        """Close page pointed by mouse cursor"""
        self._close(page)
    
    def _close(self, page_widget):
        """Close tab that contains page_widget

        In addition to call removePage() method, this method removes 
        page_widget from _model_view_registry.
        """
        self.removePage(page_widget)
        self._model_view_registry.remove(page_widget)
        page_widget.close()

    def _reload_current_page(self):
        self._reload(self.current_board())

    def _reload_pointed_page(self, page):
        board = self._model_view_registry.model(page)
        if not board is None:
            self._reload(board)

    def _reload(self, board):
        if board!=None:
            self.emit(PYSIGNAL("requestReloadTopicList"), (board,))

    def _create_list_view(self):
        """Create and return topic list view

        This method create the widget and connect signal/slot as well.
        """
        # Create widget
        view = QListView(self)
        view.setAllColumnsShowFocus(True)
        view.setSorting(0, True)
        view.addColumn("No")
        view.addColumn(u("タイトル"))
        view.addColumn(u("レス"))
        view.addColumn(u("既得"))
        # Connect signal/slot
        self.connect(view,
                     SIGNAL("doubleClicked(QListViewItem *)"),
                     self._request_open)
        self.connect(view,
                     SIGNAL("returnPressed(QListViewItem *)"),
                     self._request_open)
        self.connect(
              view,
              SIGNAL("rightButtonClicked(QListViewItem *,const QPoint &, int)"),
              self._popup_context_menu)
        return view

    class _ListViewItem(QListViewItem):
        def __init__(self, topic, *args):
            QListViewItem.__init__(self, *args)
            self._topic = topic

        def topic(self):
            return self._topic

    class _ContextMenu(QPopupMenu):
        def __init__(self, *args):
            QPopupMenu.__init__(self, *args)
            self._signal = {}
            for text, signal in [(u("新しいタブで開く"),
                                  "requestOpenInNewTab"),
                                 (u("今のタブで開く"),
                                  "requestOpenInCurrentTab"),
                                 (u("ブックマークする"),
                                  "requestBookmark")]:
                item_id = self.insertItem(text)
                self._signal[item_id] = signal
            self._item = None
            self.connect(self, SIGNAL("activated(int)"), self._activated)

        def _activated(self, item_id):
            self.emit(PYSIGNAL(self._signal[item_id]), (self._item,))

        def popup(self, item, position):
            self._item = item
            QPopupMenu.popup(self, position)
        
